/* evilwm - Minimalist Window Manager for X
 * Copyright (C) 1999-2000 Ciaran Anscomb <kja3@atdot.org>
 * see README for license and other details. */

#include "evilwm.h"
#include <X11/Xproto.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>

void do_event_loop()
{
	XEvent ev;

	for (;;) {
		XNextEvent(dpy, &ev);
#ifdef DEBUG
		show_event(ev);
#endif
		switch (ev.type) {
			case KeyPress:
				handle_key_event(&ev.xkey); break;
			case ButtonPress:
				handle_button_event(&ev.xbutton); break;
			case ConfigureRequest:
				handle_configure_request(&ev.xconfigurerequest); break;
			case MapRequest:
				handle_map_request(&ev.xmaprequest); break;
			case UnmapNotify:
				handle_unmap_event(&ev.xunmap); break;
			case ClientMessage:
				handle_client_message(&ev.xclient); break;
			case ColormapNotify:
				handle_colormap_change(&ev.xcolormap); break;
			case PropertyNotify:
				handle_property_change(&ev.xproperty); break;
			case EnterNotify:
				handle_enter_event(&ev.xcrossing); break;
			/* case Expose:
				handle_expose_event(&ev.xexpose); break; */
		}
	}
}

void spawn(char *cmd) {
	pid_t pid = fork();

	if (pid == 0)
		execlp(cmd, cmd, NULL);
}

void handle_signal(int signo) {
	if (signo == SIGCHLD) {
		wait(NULL);
		return;
	}
	quit_nicely();
}

void quit_nicely(void) {
	while(head_client) remove_client(head_client, QUITTING);
	XSetInputFocus(dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
	XInstallColormap(dpy, DefaultColormap(dpy, screen));
	XCloseDisplay(dpy);
	exit(0);
}

int handle_xerror(Display *dpy, XErrorEvent *e)
{
	Client *c = find_client(e->resourceid);

	/* if (e->error_code == BadAccess && e->resourceid == root) { */
	if (e->error_code == BadAccess && e->request_code == X_ChangeWindowAttributes) {
		fprintf(stderr, "root window unavailable (maybe another wm is running?)\n");
		exit(1);
	}
#ifdef DEBUG
	else {
		char msg[80];
		XGetErrorText(dpy, e->error_code, msg, sizeof(msg));
		fprintf(stderr, "X error (%#x): %s\n", e->resourceid, msg);
	}
#endif
	if (c) remove_client(c, NOT_QUITTING);
	return 0;
}

#ifdef DEBUG
void show_event(XEvent e)
{
	char *s;
	Window w;

	switch (e.type) {
		case ButtonPress:
			s = "ButtonPress"; w = e.xbutton.window; break;
		case ButtonRelease:
			s = "ButtonRelease"; w = e.xbutton.window; break;
		case ClientMessage:
			s = "ClientMessage"; w = e.xclient.window; break;
		case ColormapNotify:
			s = "ColormapNotify"; w = e.xcolormap.window; break;
		case ConfigureNotify:
			s = "ConfigNotify"; w = e.xconfigure.window; break;
		case ConfigureRequest:
			s = "ConfigRequest"; w = e.xconfigurerequest.window; break;
		case CreateNotify:
			s = "CreateNotify"; w = e.xcreatewindow.window; break;
		case DestroyNotify:
			s = "DestroyNotify"; w = e.xdestroywindow.window; break;
		case EnterNotify:
			s = "EnterNotify"; w = e.xcrossing.window; break;
		case Expose:
			s = "ExposeEvent"; w = e.xexpose.window; break;
		case MapNotify:
			s = "MapNotify"; w = e.xmap.window; break;
		case MapRequest:
			s = "MapRequest"; w = e.xmaprequest.window; break;
		case MappingNotify:
			s = "MappingNotify"; w = e.xmapping.window; break;
		case MotionNotify:
			s = "MotionNotify"; w = e.xmotion.window; break;
		case PropertyNotify:
			s = "PropertyNotify"; w = e.xproperty.window; break;
		case ReparentNotify:
			s = "ReparentNotify"; w = e.xreparent.window; break;
		case ResizeRequest:
			s = "ResizeRequest"; w = e.xresizerequest.window; break;
		case UnmapNotify:
			s = "UnmapNotify"; w = e.xunmap.window; break;
		default:
			s = "unknown event"; w = None; break;
	}

	fprintf(stderr, "recieved %s,\twindow = %#x\n", s, w);
}

void dump_clients()
{
	Client *c;

	for (c = head_client; c; c = c->next)
		fprintf(stderr, "(%d, %d): %#x (%#x) @ %d,%d\n", wm_state(c),
			c->ignore_unmap, c->window, c->parent, c->x, c->y);
}
#endif
